
#property indicator_separate_window
#property indicator_buffers    6
#property indicator_color1     LimeGreen
#property indicator_color2     MediumVioletRed
#property indicator_color3     MediumVioletRed
#property indicator_color4     DeepSkyBlue
#property indicator_color5     DeepSkyBlue
#property indicator_color6     Red
#property indicator_width1     2
#property indicator_width2     2
#property indicator_width4     2
#property indicator_style6     STYLE_DASH
#property indicator_levelcolor DarkSlateGray

//
//
//
//
//

extern string TimeFrame          = "Current time frame";
extern double filter             = 1.0;
extern double cycles             = 5.0;
extern double mafilter           = 1.0;
extern double macycles           = 0.25;
extern int    SmoothLength       = 50;
extern double SmoothPhase        = 0;
extern bool   SmoothDouble       = false;
extern bool   ShowHistogram      = true;
extern double levelOs            = -40.0;
extern double levelOb            = 40;
extern bool   Interpolate        = true;

extern bool   alertsOn           = false;
extern bool   alertsOnSlope      = false;
extern bool   alertsOnCurrent    = true;
extern bool   alertsMessage      = true;
extern bool   alertsSound        = false;
extern bool   alertsEmail        = false;

extern bool   arrowsVisible      = false;
extern bool   arrowsOnSlope      = false;
extern string arrowsIdentifier   = "Wpr Arrows1";
extern double arrowsDisplacement = 1.0;
extern color  arrowsUpColor      = DeepSkyBlue;
extern color  arrowsDnColor      = PaleVioletRed;
extern int    arrowsUpCode       = 241;
extern int    arrowsDnCode       = 242;

//
//
//
//
//

double wpr[];
double Upa[];
double Upb[];
double Dna[];
double Dnb[];
double wprma[];
double trend[];
double slope[];

//
//
//
//
//

string indicatorFileName;
bool   returnBars;
bool   calculateValue;
int    timeFrame;

//+------------------------------------------------------------------
//|                                                                  
//+------------------------------------------------------------------
// 
//
//
//
//

int init()
{
    IndicatorBuffers(8);
    SetIndexBuffer(0,wpr);
    SetIndexBuffer(1,Dna);
    SetIndexBuffer(2,Dnb);
    SetIndexBuffer(3,Upa);
    SetIndexBuffer(4,Upb);
    SetIndexBuffer(5,wprma);
    SetIndexBuffer(6,trend);
    SetIndexBuffer(7,slope);
    SetLevelValue(0,0);
    SetLevelValue(1,levelOs);
    SetLevelValue(2,levelOb);
   
    if (ShowHistogram)
    {
      SetIndexStyle(1,DRAW_HISTOGRAM); SetIndexLabel(1,"smooth %R");
      SetIndexStyle(2,DRAW_HISTOGRAM); SetIndexLabel(2,"smooth %R");
      SetIndexStyle(3,DRAW_HISTOGRAM); SetIndexLabel(3,"smooth %R");
      SetIndexStyle(4,DRAW_HISTOGRAM); SetIndexLabel(4,"smooth %R");
    }
    else
    {
      SetIndexStyle(1,DRAW_NONE); SetIndexLabel(1,NULL);
      SetIndexStyle(2,DRAW_NONE); SetIndexLabel(2,NULL);
      SetIndexStyle(3,DRAW_NONE); SetIndexLabel(3,NULL);
      SetIndexStyle(4,DRAW_NONE); SetIndexLabel(4,NULL);
    }
    
    //
    //
    //
    //
    //
   
      indicatorFileName = WindowExpertName();
      returnBars        = (TimeFrame == "returnBars");     if (returnBars)     return(0);
      calculateValue    = (TimeFrame == "calculateValue"); if (calculateValue) return(0);
      timeFrame         = stringToTimeFrame(TimeFrame);   

    //
    //
    //
    //
    //
    
  
    IndicatorShortName(timeFrameToString(timeFrame)+"  smooth pa %R ("+DoubleToStr(cycles,2)+","+DoubleToStr(filter,2)+","+DoubleToStr(macycles,2)+","+DoubleToStr(mafilter,2)+")");

return(0);
}

//
//
//
//
//

int deinit() 
{ 
   if (!calculateValue && arrowsVisible) deleteArrows();  
return(0); 
}

//
//
//
//
//

int start()
{
   int i,counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
         int limit = MathMin(Bars-counted_bars,Bars-1);
         if (returnBars) { wpr[0] = limit+1; return(0); }

   //
   //
   //
   //
   //
   
   if (calculateValue || timeFrame == Period())
   {
      for (i=limit; i>=0; i--)
      {
          double hi  = High[iHighest(NULL,0,MODE_HIGH,iHilbertPhase(iMA(NULL,0,1,0,MODE_SMA,PRICE_MEDIAN,i),filter,cycles,i,0),i)];
          double lo  =  Low[iLowest(NULL,  0,MODE_LOW,iHilbertPhase(iMA(NULL,0,1,0,MODE_SMA,PRICE_MEDIAN,i),filter,cycles,i,1),i)];         
          if (hi!=lo)
               wpr[i] = iDSmooth(50+(-100)*(hi - Close[i]) /(hi - lo),SmoothLength,SmoothPhase,SmoothDouble,i);
          else wpr[i] = iDSmooth(0,                                   SmoothLength,SmoothPhase,SmoothDouble,i);    
                      
      }
                 
      //
      //
      //
      //
      //
                
      for (i=limit; i>=0; i--)
      {
          wprma[i] = iMAOnArray(wpr,0,iHilbertPhase(iMA(NULL,0,1,0,MODE_SMA,PRICE_MEDIAN,i),mafilter,macycles,i,2),0,MODE_SMA,i);
          Dna[i]   = EMPTY_VALUE;
          Dnb[i]   = EMPTY_VALUE;
          Upa[i]   = EMPTY_VALUE;
          Upb[i]   = EMPTY_VALUE;
          trend[i] = trend[i+1];
          slope[i] = slope[i+1];
            if (wpr[i] > 0)        trend[i] =  1;
            if (wpr[i] < 0)        trend[i] = -1;
            if (wpr[i] > wpr[i+1]) slope[i] =  1;
            if (wpr[i] < wpr[i+1]) slope[i] = -1;
            manageArrow(i);  
                    
            //
            //
            //
            //
            //
             
            if (ShowHistogram)
            {
              if (trend[i]== 1 && slope[i] == 1) Upa[i] = wpr[i];
              if (trend[i]== 1 && slope[i] ==-1) Upb[i] = wpr[i];
              if (trend[i]==-1 && slope[i] ==-1) Dna[i] = wpr[i];
              if (trend[i]==-1 && slope[i] == 1) Dnb[i] = wpr[i];
            }
                   
      }
      manageAlerts();
     return(0);
     }

     //
     //
     //
     //
     //

     limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
    
     for (i=limit; i>=0; i--)
     {
        int y = iBarShift(NULL,timeFrame,Time[i]);
           wpr[i]   = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",filter,cycles,mafilter,macycles,SmoothLength,SmoothPhase,SmoothDouble,ShowHistogram,levelOs,levelOb,0,y);
           wprma[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",filter,cycles,mafilter,macycles,SmoothLength,SmoothPhase,SmoothDouble,ShowHistogram,levelOs,levelOb,5,y);
           trend[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",filter,cycles,mafilter,macycles,SmoothLength,SmoothPhase,SmoothDouble,ShowHistogram,levelOs,levelOb,6,y);
           slope[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",filter,cycles,mafilter,macycles,SmoothLength,SmoothPhase,SmoothDouble,ShowHistogram,levelOs,levelOb,7,y);
           Dna[i]   = EMPTY_VALUE;
           Dnb[i]   = EMPTY_VALUE;
           Upa[i]   = EMPTY_VALUE;
           Upb[i]   = EMPTY_VALUE;
           
           manageArrow(i);
           
           if (ShowHistogram)
           {
             if (trend[i]== 1 && slope[i] == 1) Upa[i] = wpr[i];
             if (trend[i]== 1 && slope[i] ==-1) Upb[i] = wpr[i];
             if (trend[i]==-1 && slope[i] ==-1) Dna[i] = wpr[i];
             if (trend[i]==-1 && slope[i] == 1) Dnb[i] = wpr[i];
   
           }
           
           //
           //
           //
           //
           //
           
           if (!Interpolate || y==iBarShift(NULL,timeFrame,Time[i-1])) continue;

           //
           //
           //
           //
           //

           datetime time = iTime(NULL,timeFrame,y);
              for(int n = 1; i+n < Bars && Time[i+n] >= time; n++) continue;
              for(int k = 1; k < n; k++)
              {
                 wpr[i+k]   = wpr[i]   +(wpr[i+n]    - wpr[i])   * k/n;
                 wprma[i+k] = wprma[i] +(wprma[i+n]  - wprma[i]) * k/n;
                 if (Upa[i]!= EMPTY_VALUE) Upa[i+k]  = wpr[i+k];
                 if (Upb[i]!= EMPTY_VALUE) Upb[i+k]  = wpr[i+k];
                 if (Dna[i]!= EMPTY_VALUE) Dna[i+k]  = wpr[i+k];
                 if (Dnb[i]!= EMPTY_VALUE) Dnb[i+k]  = wpr[i+k];
              }
     }
     manageAlerts();
return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = stringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}

//
//
//
//
//

string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string stringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int tchar = StringGetChar(s, length);
         if((tchar > 96 && tchar < 123) || (tchar > 223 && tchar < 256))
                     s = StringSetChar(s, length, tchar - 32);
         else if(tchar > -33 && tchar < 0)
                     s = StringSetChar(s, length, tchar + 224);
   }
   return(s);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

double wrk[][20];

#define bsmax  5
#define bsmin  6
#define volty  7
#define vsum   8
#define avolty 9

//
//
//
//
//

double iDSmooth(double price, double length, double phase, bool isDouble, int i, int s=0)
{
   if (isDouble)
         return (iSmooth(iSmooth(price,MathSqrt(length),phase,i,s),MathSqrt(length),phase,i,s+10));
   else  return (iSmooth(price,length,phase,i,s));
}

//
//
//
//
//

double iSmooth(double price, double length, double phase, int i, int s=0)
{
   if (length <=1) return(price);
   if (ArrayRange(wrk,0) != Bars) ArrayResize(wrk,Bars);
   
   int r = Bars-i-1; 
      if (r==0) { for(int k=0; k<7; k++) wrk[r][k+s]=price; for(; k<10; k++) wrk[r][k+s]=0; return(price); }

   //
   //
   //
   //
   //
   
      double len1   = MathMax(MathLog(MathSqrt(0.5*(length-1)))/MathLog(2.0)+2.0,0);
      double pow1   = MathMax(len1-2.0,0.5);
      double del1   = price - wrk[r-1][bsmax+s];
      double del2   = price - wrk[r-1][bsmin+s];
      double div    = 1.0/(10.0+10.0*(MathMin(MathMax(length-10,0),100))/100);
      int    forBar = MathMin(r,10);
	
         wrk[r][volty+s] = 0;
               if(MathAbs(del1) > MathAbs(del2)) wrk[r][volty+s] = MathAbs(del1); 
               if(MathAbs(del1) < MathAbs(del2)) wrk[r][volty+s] = MathAbs(del2); 
         wrk[r][vsum+s] =	wrk[r-1][vsum+s] + (wrk[r][volty+s]-wrk[r-forBar][volty+s])*div;
         
         //
         //
         //
         //
         //
   
         wrk[r][avolty+s] = wrk[r-1][avolty+s]+(2.0/(MathMax(4.0*length,30)+1.0))*(wrk[r][vsum+s]-wrk[r-1][avolty+s]);
            if (wrk[r][avolty+s] > 0)
               double dVolty = wrk[r][volty+s]/wrk[r][avolty+s]; else dVolty = 0;   
	               if (dVolty > MathPow(len1,1.0/pow1)) dVolty = MathPow(len1,1.0/pow1);
                  if (dVolty < 1)                      dVolty = 1.0;

      //
      //
      //
      //
      //
	        
   	double pow2 = MathPow(dVolty, pow1);
      double len2 = MathSqrt(0.5*(length-1))*len1;
      double Kv   = MathPow(len2/(len2+1), MathSqrt(pow2));

         if (del1 > 0) wrk[r][bsmax+s] = price; else wrk[r][bsmax+s] = price - Kv*del1;
         if (del2 < 0) wrk[r][bsmin+s] = price; else wrk[r][bsmin+s] = price - Kv*del2;
	
   //
   //
   //
   //
   //
      
      double R     = MathMax(MathMin(phase,100),-100)/100.0 + 1.5;
      double beta  = 0.45*(length-1)/(0.45*(length-1)+2);
      double alpha = MathPow(beta,pow2);

         wrk[r][0+s] = price + alpha*(wrk[r-1][0+s]-price);
         wrk[r][1+s] = (price - wrk[r][0+s])*(1-beta) + beta*wrk[r-1][1+s];
         wrk[r][2+s] = (wrk[r][0+s] + R*wrk[r][1+s]);
         wrk[r][3+s] = (wrk[r][2+s] - wrk[r-1][4+s])*MathPow((1-alpha),2) + MathPow(alpha,2)*wrk[r-1][3+s];
         wrk[r][4+s] = (wrk[r-1][4+s] + wrk[r][3+s]); 

   //
   //
   //
   //
   //

   return(wrk[r][4+s]);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

double workHil[][27];
#define _price      0
#define _smooth     1
#define _detrender  2
#define _period     3
#define _instPeriod 4
#define _phase      5
#define _deltaPhase 6
#define _Q1         7
#define _I1         8

#define Pi 3.14159265358979323846264338327950288

//
//
//
//
//

double iHilbertPhase(double price, double filter1, double cyclesToReach, int i, int s=0)
{
   if (ArrayRange(workHil,0)!=Bars) ArrayResize(workHil,Bars);
   int r = Bars-i-1; s = s*9;
      
   //
   //
   //
   //
   //
      
      workHil[r][s+_price]      = price;
      workHil[r][s+_smooth]     = (4.0*workHil[r][s+_price]+3.0*workHil[r-1][s+_price]+2.0*workHil[r-2][s+_price]+workHil[r-3][s+_price])/10.0;
      workHil[r][s+_detrender]  = calcComp(r,_smooth,s);
      workHil[r][s+_Q1]         = 0.15*calcComp(r,_detrender,s)  +0.85*workHil[r-1][s+_Q1];
      workHil[r][s+_I1]         = 0.15*workHil[r-3][s+_detrender]+0.85*workHil[r-1][s+_I1];
      workHil[r][s+_phase]      = workHil[r-1][s+_phase];
      workHil[r][s+_instPeriod] = workHil[r-1][s+_instPeriod];

      //
      //
      //
      //
      //
           
         if (MathAbs(workHil[r][s+_I1])>0)
                     workHil[r][s+_phase] = 180.0/Pi*MathArctan(MathAbs(workHil[r][s+_Q1]/workHil[r][s+_I1]));
           
         if (workHil[r][s+_I1]<0 && workHil[r][s+_Q1]>0) workHil[r][s+_phase] = 180-workHil[r][s+_phase];
         if (workHil[r][s+_I1]<0 && workHil[r][s+_Q1]<0) workHil[r][s+_phase] = 180+workHil[r][s+_phase];
         if (workHil[r][s+_I1]>0 && workHil[r][s+_Q1]<0) workHil[r][s+_phase] = 360-workHil[r][s+_phase];

      //
      //
      //
      //
      //
                        
      workHil[r][s+_deltaPhase] = workHil[r-1][s+_phase]-workHil[r][s+_phase];

         if (workHil[r-1][s+_phase]<90 && workHil[r][s+_phase]>270)
             workHil[r][s+_deltaPhase] = 360+workHil[r-1][s+_phase]-workHil[r][s+_phase];
             workHil[r][s+_deltaPhase] = MathMax(MathMin(workHil[r][s+_deltaPhase],60),7);
      
            //
            //
            //
            //
            //
                  
            double alpha    = 2.0/(1.0+MathMax(filter1,1));
            double phaseSum = 0; for (int k=0; phaseSum<cyclesToReach*360 && (r-k)>0; k++) phaseSum += workHil[r-k][s+_deltaPhase];
         
            if (k>0) workHil[r][s+_instPeriod]= k;
                    workHil[r][s+_period] = workHil[r-1][s+_period]+alpha*(workHil[r][s+_instPeriod]-workHil[r-1][s+_period]);
            return (workHil[r][s+_period]);
}

//
//
//
//
//

double calcComp(int r, int from, int s)
{
   return((0.0962*workHil[r  ][s+from] + 
           0.5769*workHil[r-2][s+from] - 
           0.5769*workHil[r-4][s+from] - 
           0.0962*workHil[r-6][s+from]) * (0.075*workHil[r-1][s+_period] + 0.54));
}

//
//
//
//
//

void manageAlerts()
{
   if (!calculateValue && alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1; whichBar = iBarShift(NULL,0,iTime(NULL,timeFrame,whichBar));
      
      //
      //
      //
      //
      //
      
      if (alertsOnSlope)
      {
         if (slope[whichBar] != slope[whichBar+1])
         {
            if (slope[whichBar] == 1) doAlert(whichBar,"slope changed to up");
            if (slope[whichBar] ==-1) doAlert(whichBar,"slope changed to down");
         }         
      }
      else
      {
         if (trend[whichBar] != trend[whichBar+1])
         {
            if (trend[whichBar] == 1) doAlert(whichBar,"crossed zero line up");
            if (trend[whichBar] ==-1) doAlert(whichBar,"crossed zero line down");
         }         
      }         
   }
}   

//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
      if (previousAlert != doWhat || previousTime != Time[forBar]) {
          previousAlert  = doWhat;
          previousTime   = Time[forBar];

          //
          //
          //
          //
          //

          message =  StringConcatenate(Symbol()," ",timeFrameToString(timeFrame)," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," Wpr oscillator ",doWhat);
             if (alertsMessage) Alert(message);
             if (alertsEmail)   SendMail(StringConcatenate(Symbol(),"Wpr"),message);
             if (alertsSound)   PlaySound("alert2.wav");
      }
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

void manageArrow(int i)
{
   if (arrowsVisible)
   {
      deleteArrow(Time[i]);
      if(arrowsOnSlope)
      {
      
         if (slope[i] != slope[i+1])
         {
            if (slope[i] == 1) drawArrow(i,arrowsUpColor,arrowsUpCode,false);
            if (slope[i] ==-1) drawArrow(i,arrowsDnColor,arrowsDnCode, true);
         }         
      }
      else
      {
         if (trend[i] != trend[i+1])
         {
            if (trend[i] == 1) drawArrow(i,arrowsUpColor,arrowsUpCode,false);
            if (trend[i] ==-1) drawArrow(i,arrowsDnColor,arrowsDnCode, true);
         }  
      }         
   }         
}       

//
//
//
//
//

void drawArrow(int i,color theColor,int theCode,bool up)
{
   string name = arrowsIdentifier+":"+Time[i];
   double gap  = iATR(NULL,0,20,i);   
   
      //
      //
      //
      //
      //
      
      ObjectCreate(name,OBJ_ARROW,0,Time[i],0);
         ObjectSet(name,OBJPROP_ARROWCODE,theCode);
         ObjectSet(name,OBJPROP_COLOR,theColor);
         if (up)
               ObjectSet(name,OBJPROP_PRICE1,High[i] + arrowsDisplacement * gap);
         else  ObjectSet(name,OBJPROP_PRICE1,Low[i]  - arrowsDisplacement * gap);
}

//
//
//
//
//

void deleteArrows()
{
   string lookFor       = arrowsIdentifier+":";
   int    lookForLength = StringLen(lookFor);
   for (int i=ObjectsTotal()-1; i>=0; i--)
   {
      string objectName = ObjectName(i);
         if (StringSubstr(objectName,0,lookForLength) == lookFor) ObjectDelete(objectName);
   }
}

//
//
//
//
//

void deleteArrow(datetime time)
{
   string lookFor = arrowsIdentifier+":"+time; ObjectDelete(lookFor);
}